##Nov. 27, 2012
#R code for Livingston et al.##
################ Load Packages ###############
library(moments)
library(gdata)
library(lattice)
############################ Read in Data ############################
setwd("~/Desktop/R input files")  # Set working directory

# Read in data 
mydata<-read.csv("~/Desktop/R input files/Emerge/data_07062011.csv",header=TRUE) # "data_07062011.csv" is the raw file of each species' occurrence per community pre- and post- mixing
mydata.pre.ori<-mydata[mydata$sample=="Pre",] #initial communities (subset of mydata)
mydata.post.ori<-mydata[mydata$sample=="Post",] # species composition at the time of mixing (subset of mydata)
mix.com.name<-names(summary(mydata$mixcommunity))[-1] # # Retrieve the names of all mixed communities

persistance_pr<-read.csv("~/Desktop/R input files/Emerge/persistance_prob.csv",header=TRUE) # "persistance_prob" is species' persistence probability


connect_deviant_data<-read.csv("~/Desktop/R input files/Emerge/composition asymmetry.csv",header=TRUE) # Observed asymmetries

simmulation.no<-10 # The number of simulation runs

############################################################################################ Asymmetry in Species Composition#############

################################# Define Functions ##################
# 1) Function: Asym.Comp() 
# Function Asym.Comp() takes in two datasets, "mydata.mix" (communities present after simulated extinctions) and "mydata.post.ori" (communities present at the time of mixing). This function is used to calculate the overall asymmetry in species composition and asymmetry in species composition in each community by guild.

Asym.Comp<-function (mydata.mix,mydata.post.ori) {
  # Create null vectors
  Diff<-c()
  exC1<-c()
  exC2<-c()
  S1<-c()
  S2<-c()
  
  #Retrieve the name of the two mixed communities in each "mix" communities pairs
  for (i in 1:length(mix.com.name)){
    mix1.com<-mydata.mix[mydata.mix$mixcommunity==mix.com.name[i],]$mix1[1]  # Retrieve the name of the first pre-mix community
    mix2.com<-mydata.mix[mydata.mix$mixcommunity==mix.com.name[i],]$mix2[1] # Retrieve the name of the second pre-mix community
    
    # Asymmetry Calculation
    exC1<-length(mydata.post.ori[mydata.post.ori$community==mix1.com,]$sppID) - length(mydata.mix[mydata.mix$community==mix1.com,]$sppID) # exC1=simulated number of species that went extinct post-assembly in the first premixing community
    exC2<-length(mydata.post.ori[mydata.post.ori$community==mix2.com,]$sppID) - length(mydata.mix[mydata.mix$community==mix2.com,]$sppID) # exC2=simulated number of species that went extinct post-assembly in the second premixing community
    S1<-length(mydata.post.ori[mydata.post.ori$community==mix1.com,]$sppID)  #S1= the number of species present at the time of mixing in the first premixing community
    S2<-length(mydata.post.ori[mydata.post.ori$community==mix2.com,]$sppID)  #S2= the number of species present at the time of mixing in the second premixing community
    
    Diff.temp<-abs((exC1/S1)-(exC2/S2))  # Asymmetry: the absolute difference in extinction rates
    Diff<-cbind(Diff, as.numeric(Diff.temp)) # Diff contains the asymmetry values of each mixed community in each simulation run
  }
  return(Diff)
}
##### the end of function Asym.Comp() #########

# 2) Function: Asym.Comp.Treat()

# Function Asym.Comp.Treat() takes in three datasets, "mydata.mix" (simulated data: communities present after mixing), "mydata.post.ori" (experimental data: communities present at the time of mixing) and "mix.com.name" (a vector of names of all mixed communities). this function calculates the asymmetry in species composition in each community by treatment.

Asym.Comp.Treat<-function (mydata.mix,mydata.post.ori,mix.com.name) 
{
  # Create null vectors
  Diff<-c()
  exC1<-c()
  exC2<-c()
  S1<-c()
  S2<-c()
  
  #Retrieve the name of the two mixed communities in each "mix" pair of communities
  for (i in 1:length(mix.com.name))
  {
    mix1.com<-mydata.mix[mydata.mix$mixcommunity==mix.com.name[i],]$mix1[1]  # Retrieve the name of the first pre-mix community
    mix2.com<-mydata.mix[mydata.mix$mixcommunity==mix.com.name[i],]$mix2[1]  # Retrieve the name of the second pre-mix community
    
    # Asymmetry Calculation:
    # exC1/exC2=simulated number of species went extinct post-assembly in each of the premixing communities; S1/S2= the number of species actually present at the time of mixing in each of the premixing communities
    exC1 <- length(mydata.post.ori[mydata.post.ori$community==mix1.com,]$sppID) - length(mydata.mix[mydata.mix$community==mix1.com,]$sppID)
    exC2 <- length(mydata.post.ori[mydata.post.ori$community==mix2.com,]$sppID) - length(mydata.mix[mydata.mix$community==mix2.com,]$sppID)	
    S1<-length(mydata.post.ori[mydata.post.ori$community==mix1.com,]$sppID)
    S2<-length(mydata.post.ori[mydata.post.ori$community==mix2.com,]$sppID)
    Diff.temp<-abs((exC1/S1)-(exC2/S2)) # Asymmetry: the absolute difference in extinction rates
    Diff<-cbind(Diff, as.numeric(Diff.temp)) # Diff contains the asymmetries of each mixed community in each simulated dataset
  }
  return(Diff)
}
##### the end of function Asym.Comp.Treat() #########

# 3) Function: Asym.Comp.Stat()

# Function Asym.Comp.Stat() takes in a dataset "Asymmetry.t" (each row of Asymmetry.t contains simulated asymmetries in species compostition in each mixed communites in one simulation run) and calculate an overall mean asymmetry across all communities across all simulation runs, with standard deviation and 95% confidence interval.

Asym.Comp.Stat <- function (Asymmetry.t) {
  # transpose dataset
  Asymmetry<-t(Asymmetry.t)
  
  # Create null vectors
  asymmetry_mean_total<-c()
  asymmetry_mean_total_temp<-c()
  asymmetry_sd_total<-c()
  asymmetry_low_total<-c()
  asymmetry_high_total<-c()
  Difference_total<-c()
  
  # Extract mean asymmetry of each communities (a total of 66 mean asymmetries) across all simulation runs for following calculations
  for (i in 1:simmulation.no) {     
    asymmetry_mean_total_temp <-cbind(asymmetry_mean_total_temp, mean(Asymmetry[,i], na.rm=TRUE))
  }
  
  asymmetry_mean_total<-mean(as.numeric(asymmetry_mean_total_temp),na.rm=TRUE) #calculate the overall mean asymmetry
  asymmetry_sd_total<-sd(as.numeric(asymmetry_mean_total_temp),na.rm=TRUE) #calculate SD 
  asymmetry_low_total<-qnorm(0.025,mean= asymmetry_mean_total,sd=asymmetry_sd_total) #calculate the lower boundary of 95% CI
  asymmetry_high_total<-qnorm(0.975,mean= asymmetry_mean_total,sd=asymmetry_sd_total)  #calculate the upper boundary of 95% CI
  #Export data
  Difference_total <- as.data.frame(cbind(t(asymmetry_mean_total), t(asymmetry_sd_total), t(asymmetry_low_total), t(asymmetry_high_total)))
  colnames(Difference_total)<-c("mean","SD","95% CI","95% CI")
  return(Difference_total) 
}
###### Function "Asym.Comp.Stat.Var" Ends ######
.

# 4) Function: Asym.Comp.Stat.Var() 
# Function Asym.Comp.Stat.Var() takes in a dataset "Asymmetry.t" (which contains all simulated asymmetries in species compostition in each mixed communites in each simulation run) and calculate an overall mean variance of asymmetry across all communities, with standard deviation and 95% confidence interval.

Asym.Comp.Stat.Var <- function (Asymmetry.t) {
  # transpose dataset
  Asymmetry<-t(Asymmetry.t)
  
  # Create null vectors
  asymmetry_var_total<-c()
  asymmetry_var_total_temp<-c()
  asymmetry_sd_var_total<-c()
  asymmetry_low_var_total<-c()
  asymmetry_high_var_total<-c()
  Difference_var_total<-c()
  
  # Extract the variance in asymmetry of all communities in each simulation run for following calculations
  for (i in 1:simmulation.no) {
    asymmetry_var_total_temp<-cbind(asymmetry_var_total_temp,var(Asymmetry[,i],na.rm=TRUE))
  } 
  
  asymmetry_var_total<-mean(as.numeric(asymmetry_var_total_temp),na.rm=TRUE) #calculate the overall mean variance of asymmetry
  asymmetry_sd_var_total<-sd(as.numeric(asymmetry_var_total_temp),na.rm=TRUE) #calculate SD
  asymmetry_low_var_total<- qnorm(0.025,mean=asymmetry_var_total, sd=asymmetry_sd_var_total)  #calculate the lower boundary of 95% CI
  asymmetry_high_var_total<-qnorm(0.975,mean= asymmetry_var_total,sd=asymmetry_sd_var_total)  #calculate the upper boundary of 95% CI
  
  #Export data
  Difference_var_total<-as.data.frame (cbind(t(asymmetry_var_total), t(asymmetry_sd_var_total), t(asymmetry_low_var_total), t(asymmetry_high_var_total)))
  colnames(Difference_var_total)<-c("mean","SD","95% CI","95% CI")
  return(Difference_var_total) 
}

######### Function "Asym.Comp.Stat.Var" Ends ########



#############################Simulations#################################

# Create null vectors for overall asymmetry in species composition
mydata.left<-c()
Asymmetry.t<-c()

# Create null vectors for asymmetry in species compostition by guild
mydata.mix.Bac<-c()
Asymmetry.t.Bac<-c()
mydata.mix.Edible<-c()
Asymmetry.t.Edible<-c()
mydata.mix.Herb<-c()
Asymmetry.t.Herb<-c()
mydata.mix.Inedible<-c()
Asymmetry.t.Inedible<-c()
mydata.mix.Omni<-c()
Asymmetry.t.Omni<-c()
mydata.mix.Predator<-c()
Asymmetry.t.Predator<-c()

# Create null vectors for asymmetry in species compostition by treatment 
mydata.mix.1_2<-c()
Asymmetry.t.1_2<-c()
mydata.mix.3<-c()
Asymmetry.t.3<-c()
mydata.mix.CC<-c()
Asymmetry.t.CC<-c()
mydata.mix.TC<-c()
Asymmetry.t.TC<-c()
mydata.mix.TT<-c()
Asymmetry.t.TT<-c()

for (i in 1: simmulation.no) {
  cat("simulation",i,"\n") #Print current simulation number
  persist<-c() 
  mydata.left<-c()
  #The following "for loop"" finds the persistence prob of each species in "mydata", and simulates whether it will persist or not and return the result as TRUE/FALSE
  for (i in 1:length(mydata$species)) {
    p<-c()
    p<-c(p,persistance_pr[persistance_pr[1]==as.character(mydata$species[i]),2]) # For each species in "mydata", find its persistance probability in "persistance_pr" and save it as p
    persist.temp<-sample(c(TRUE,FALSE),size=1,replace=TRUE,prob=c(p,1-p))   #based on persistance probability, estimate whether each sample from each species would persist or not
    persist<-c(persist,persist.temp) #save the result above as "persist"
  }
  #After simulating every species in "mydata", now "persist" is a vector with the same length as the number of rows in "mydata", and filled with TURE or FALSE
  mydata.left<-mydata[persist,] #species that persist after simulation will now be stored in a new vector "mydata.left"
  
  mydata.mix<-mydata.left[!is.na(mydata.left$mix1),]  #Extract all mix communities from "mydata.left"
  
  ######### Asymmetry in Species Composition Overall #########
  Diff<-Asym.Comp(mydata.mix,mydata.post.ori)
  Asymmetry.t<-rbind(Asymmetry.t,Diff) # Asymmetry.t contains all simulated asymmetries in species compostition in each mixed communites in each simulation run
  
  ######## Asymmetry in Species Composition by Guild#########
  # Extracting asymmetry results by guild
  #Bacteria
  mydata.mix.Bac<-mydata.mix[mydata.mix$guild=="Bac",]
  mydata.post.ori.Bac<-mydata.post.ori[mydata.post.ori$guild=="Bac",] 
  Diff.Bac<-Asym.Comp(mydata.mix.Bac,mydata.post.ori.Bac)
  Asymmetry.t.Bac<-rbind(Asymmetry.t.Bac,Diff.Bac)  
  #Edible
  mydata.mix.Edible<-mydata.mix[mydata.mix$guild=="Edible",]
  mydata.post.ori.Edible<-mydata.post.ori[mydata.post.ori$guild=="Edible",] 
  Diff.Edible<-Asym.Comp(mydata.mix.Edible,mydata.post.ori.Edible)
  Asymmetry.t.Edible<-rbind(Asymmetry.t.Edible,Diff.Edible)
  #Herbivore
  mydata.mix.Herb<-mydata.mix[mydata.mix$guild=="Herb",]
  mydata.post.ori.Herb<-mydata.post.ori[mydata.post.ori$guild=="Herb",] 
  Diff.Herb<-Asym.Comp(mydata.mix.Herb,mydata.post.ori.Herb)
  Asymmetry.t.Herb<-rbind(Asymmetry.t.Herb,Diff.Herb)
  #Inedible
  mydata.mix.Inedible<-mydata.mix[mydata.mix$guild=="Inedible ",] #"Inedible+blank"
  mydata.post.ori.Inedible<-mydata.post.ori[mydata.post.ori$guild=="Inedible ",] 
  Diff.Inedible<-Asym.Comp(mydata.mix.Inedible,mydata.post.ori.Inedible)
  Asymmetry.t.Inedible<-rbind(Asymmetry.t.Inedible,Diff.Inedible)
  #Omnivore
  mydata.mix.Omni<-mydata.mix[mydata.mix$guild=="Omni",]
  mydata.post.ori.Omni<-mydata.post.ori[mydata.post.ori$guild=="Omni",] 
  Diff.Omni<-Asym.Comp(mydata.mix.Omni,mydata.post.ori.Omni)
  Asymmetry.t.Omni<-rbind(Asymmetry.t.Omni,Diff.Omni)
  #Predator
  mydata.mix.Predator<-mydata.mix[mydata.mix$guild=="Predator",]
  mydata.post.ori.Predator<-mydata.post.ori[mydata.post.ori$guild=="Predator",] 
  Diff.Predator<-Asym.Comp(mydata.mix.Predator,mydata.post.ori.Predator)
  Asymmetry.t.Predator<-rbind(Asymmetry.t.Predator,Diff.Predator)
  
  
  
  ######## Asymmetry in Species Composition by treatment#########
  # Extracting asymmetry resuls by treatment
  # CC   
  mydata.mix.CC<-mydata.mix[mydata.mix$mixtype=="CC",]
  mydata.post.ori.CC<-mydata.post.ori[mydata.post.ori$mixtype=="CC",]
  mix.com.name.CC<-names(summary(drop.levels(mydata.post.ori.CC$mixcommunity)))
  Diff.CC<-Asym.Comp.Treat(mydata.mix.CC,mydata.post.ori.CC,mix.com.name.CC)
  Asymmetry.t.CC<-rbind(Asymmetry.t.CC,Diff.CC)
  
  # TC   
  mydata.mix.TC<-mydata.mix[mydata.mix$mixtype=="TC",]
  mydata.post.ori.TC<-mydata.post.ori[mydata.post.ori$mixtype=="TC",]
  mix.com.name.TC<-names(summary(drop.levels(mydata.post.ori.TC$mixcommunity)))
  Diff.TC<-Asym.Comp.Treat(mydata.mix.TC,mydata.post.ori.TC,mix.com.name.TC)
  Asymmetry.t.TC<-rbind(Asymmetry.t.TC,Diff.TC)
  
  # TT 
  mydata.mix.TT<-mydata.mix[mydata.mix$mixtype=="TT",]
  mydata.post.ori.TT<-mydata.post.ori[mydata.post.ori$mixtype=="TT",]
  mix.com.name.TT<-names(summary(drop.levels(mydata.post.ori.TT$mixcommunity)))
  Diff.TT<-Asym.Comp.Treat(mydata.mix.TT,mydata.post.ori.TT,mix.com.name.TT)
  Asymmetry.t.TT<-rbind(Asymmetry.t.TT,Diff.TT)  
}

# SIMULATION ENDS


#### Statistical Analysis of Asymmetry in Species Composition#####

# Mean asymmetry with SD and 95% CI
Asym.Comp.Stat(Asymmetry.t)

# Asymmetry by guild with SD and 95% CI 
Asym.Comp.Stat(Asymmetry.t.Bac)
Asym.Comp.Stat(Asymmetry.t.Edible)
Asym.Comp.Stat(Asymmetry.t.Herb)
Asym.Comp.Stat(Asymmetry.t.Inedible)
Asym.Comp.Stat(Asymmetry.t.Omni)
Asym.Comp.Stat(Asymmetry.t.Predator)

# Asymmetry by treatment with SD and 95% CI 
Asym.Comp.Stat(Asymmetry.t.CC)
Asym.Comp.Stat(Asymmetry.t.TC)
Asym.Comp.Stat(Asymmetry.t.TT)

# Overall mean variance in asymmetry with SD and 95% CI
Asym.Comp.Stat.Var(Asymmetry.t)

# Mean variance in asymmetry with SD and 95% CI by guild
Asym.Comp.Stat.Var(Asymmetry.t.Bac)
Asym.Comp.Stat.Var(Asymmetry.t.Edible)
Asym.Comp.Stat.Var(Asymmetry.t.Herb)
Asym.Comp.Stat.Var(Asymmetry.t.Inedible)
Asym.Comp.Stat.Var(Asymmetry.t.Omni)
Asym.Comp.Stat.Var(Asymmetry.t.Predator)

# Mean variance in asymmetry with SD and 95% CI by treatment
Asym.Comp.Stat.Var(Asymmetry.t.CC)
Asym.Comp.Stat.Var(Asymmetry.t.TC)
Asym.Comp.Stat.Var(Asymmetry.t.TT)

#Idividual community level deviants from 95% CI in asymmetry
# Transpose dataset
asym.comp.temp<-t(Asymmetry.t)
rownames(asym.comp.temp)<-names(summary(mydata.mix$mixcommunity))[-1]

# Create null vectors
mix_mean_indiv.comp<-c()
mix_sd_indiv.comp<-c()
mix_low_indiv.comp<-c()
mix_high_indiv.comp<-c()
connect_asym_indiv.comp<-c()

# Calculate the mean asymmetry in species composition for each mix community across all simulation runs and store the result in connect_asym_indiv.comp
for (i in 1:66) {
  mix_mean_indiv.comp<-cbind(mix_mean_indiv.comp,mean(asym.comp.temp[i,],na.rm=TRUE))
  mix_sd_indiv.comp<-cbind(mix_sd_indiv.comp,sd(asym.comp.temp[i,],na.rm=TRUE))
  mix_low_indiv.comp<-cbind(mix_low_indiv.comp,qnorm(0.00038,mean= mean(asym.comp.temp[i,],na.rm=TRUE),sd= sd(asym.comp.temp[i,],na.rm=TRUE)))  
  mix_high_indiv.comp<-cbind(mix_high_indiv.comp,qnorm(0.99962,mean= mean(asym.comp.temp[i,],na.rm=TRUE),sd=sd(asym.comp.temp[i,],na.rm=TRUE)))
}
connect_asym_indiv.comp <- as.data.frame(cbind(t(mix_mean_indiv.comp), t(mix_sd_indiv.comp), t(mix_low_indiv.comp), t(mix_high_indiv.comp)))
names(connect_asym_indiv.comp)<-c("mean","SD","95%_CI.low","95%_CI.hi")
rownames(connect_asym_indiv.comp)<-mix.com.name

#Identfy Deviants
sum(connect_deviant_data$comp_diff < connect_asym_indiv.comp[3]) #The number of deviants below CI
sum(connect_deviant_data$comp_diff > connect_asym_indiv.comp[4]) #The number of deviants above CI
